/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.app;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.http.ResultWrapper;
import net.northking.cctp.scriptcase.api.app.service.AppApiService;
import net.northking.cctp.scriptcase.dto.app.AppAddDto;
import net.northking.cctp.scriptcase.dto.app.AppUpdateDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 应用表 => Restful 类型 HTTP/HTTPS 接口
 *
 * @author maven-cctp-plugin
 * @since 1.0
 */
@Api(tags = "应用表")
@RequestMapping("/v1/batch/Apps")
@RestController
public class AppBatchCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(AppBatchCtrl.class);

  @Autowired
  private AppApiService apiService;

  /**
   * Restful接口-post： 批量新增应用表
   *
   * @param batch 集合
   * @return 主键集合
   */
  @ApiOperation(value = "批量新增：应用表")
  @PostMapping(
      value = "/insert",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<List<String>> addBatch(
      @RequestBody @Validated List<AppAddDto> batch)
  {
    ResultWrapper<List<String>> wrapper = new ResultWrapper<>();
    List<String> result = this.apiService.addBatch(batch);
    return wrapper.success(result);
  }

  /**
   * Restful接口-post： 批量修改应用表
   *
   * @param batch 集合
   * @return 成功的数量
   */
  @ApiOperation(value = "批量修改：应用表")
  @PostMapping(
      value = "/update",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> updateBatch(
      @RequestBody @Validated List<AppUpdateDto> batch) throws IllegalAccessException
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = this.apiService.updateBatch(batch);
    return wrapper.success(count);
  }

  /**
   * Restful接口-post： 批量删除应用表
   *
   * @param batch 集合
   * @return 成功的数量
   */
  @ApiOperation(value = "批量删除：应用表")
  @PostMapping(
      value = "/delete",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> deleteBatch(
      @RequestBody @Validated List<Object> batch)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = this.apiService.deleteBatch(batch);
    return wrapper.success(count);
  }
}
