/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.device.api.brand.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.device.db.entity.CdMobileBrand;
import net.northking.cctp.device.dto.brand.CdMobileBrandAddDto;
import net.northking.cctp.device.dto.brand.CdMobileBrandDetailDto;
import net.northking.cctp.device.dto.brand.CdMobileBrandQueryDto;
import net.northking.cctp.device.dto.brand.CdMobileBrandUpdateDto;

import java.util.List;

/**
 * 移动设备品牌表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface CdMobileBrandApiService extends ExcelService
{
  /**
   * 查询移动设备品牌表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<CdMobileBrandDetailDto> pagingQuery(QueryByPage<CdMobileBrandQueryDto> queryByPage);

  /**
   * 新增移动设备品牌表
   *
   * @param dto         接口数据传输对象
   * @return 数据库主键
   */
  String add(CdMobileBrandAddDto dto);

  /**
   * 更新移动设备品牌表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(CdMobileBrandUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取移动设备品牌表
   * @param uuid 数据库主键
   * @return 接口数据传输对象
   */
  CdMobileBrandDetailDto findByPK(String uuid);

  /**
   * 删除移动设备品牌表
   * @param uuid  数据库主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<CdMobileBrandAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<CdMobileBrandUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    List<CdMobileBrand> queryAllBrand();
}
