/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.device.api.mobile.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.device.db.entity.*;
import net.northking.cctp.device.dto.device.*;
import net.northking.cctp.device.dto.pcDevice.CdDeviceRemoteDto;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * 移动设备信息表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface CdMobileDeviceApiService extends ExcelService
{
  /**
   * 查询移动设备信息表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<CdMobileDeviceDetailDto> pagingQuery(QueryByPage<CdMobileDeviceQueryDto> queryByPage);

  /**
   * 新增移动设备信息表
   *
   * @param dto         接口数据传输对象
   * @return 数据库主键
   */
  String add(CdMobileDeviceAddDto dto);

  /**
   * 更新移动设备信息表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(CdMobileDeviceUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取移动设备信息表
   * @param uuid 数据库主键
   * @return 接口数据传输对象
   */
  CdMobileDeviceDetailDto findByPK(String uuid);

  /**
   * 删除移动设备信息表
   * @param uuid  数据库主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<CdMobileDeviceAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<CdMobileDeviceUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    Pagination<CdMobileDeviceDetailDto> pagingQueryByTenant(QueryByPage<CdMobileDeviceQueryDto> dto);

    Pagination<CdMobileDeviceDetailDto> pagingNoneTenant(QueryByPage<CdMobileDeviceQueryDto> dto);

    Pagination<CdMobileDeviceDetailDto> pagingNoneProject(QueryByPage<CdMobileDeviceQueryDto> dto);

    Pagination<CdMobileDeviceDetailDto> mobileByTenant(QueryByPage<CdMobileDeviceQueryDto> dto);

    Pagination<CdMobileDeviceDetailDto> mobileByProject(QueryByPage<CdMobileDeviceQueryDto> dto);

    Integer removeDevice(String id);

    CdDeviceCtrl findCtrlById(String id);

    DeviceInfoDto achieveMobDevice(CdDeviceRemoteDto dto, HttpServletRequest request);

    String restartDevice(List<String> ids);

    String registerMobile(CdDeviceRegisterDto device);

    String allocateTenant(CdMobileDeviceRecycleDto dto);

    String recycleTenant(List<String> ids);

    Integer extendTenant(CdDeviceTenant dto);

    String allocateProject(CdMobileDeviceRecycleDto ids);

    String recycleProject(List<String> ids);

    Integer extendProject(CdDeviceProject dto);

    Pagination<ScriptResultDto> scriptDeviceByPro(QueryByPage<CdMobileDeviceScriptDto> dto);

    List<Map<String,String>> remoteMobileDeviceList(Map<String,Object> deviceList, HttpServletRequest request);

    List<Map<String,String>> achieveMobileDevice(Map<String,Object> deviceList,HttpServletRequest request);

    Map<String,Object> queryCtrlIp(String uuid);

    List<DeviceInfoDto> mobGroupControl(List<String> deviceList,HttpServletRequest request);

  Pagination<ScriptResultDto> scriptDeviceByPro2(QueryByPage<CdMobileDeviceScriptDto2> queryByPage);
}
