/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.dw.api.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.dw.dto.*;

import java.util.List;

/**
 * 用例执行统计图表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface MtCaseExecChartApiService extends ExcelService
{
  /**
   * 查询用例执行统计图表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<MtCaseExecChartDetailDto> pagingQuery(QueryByPage<MtCaseExecChartQueryDto> queryByPage);

  /**
   * 新增用例执行统计图表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(MtCaseExecChartAddDto dto);

  /**
   * 更新用例执行统计图表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(MtCaseExecChartUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取用例执行统计图表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  MtCaseExecChartDetailDto findByPK(String uuid);

  /**
   * 删除用例执行统计图表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<MtCaseExecChartAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<MtCaseExecChartUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    List<MtCaseExecChartDetailDto> listQuery(ChartDataGeneQueryDto chartDataGeneQueryDto);
}
